import numpy as np
import yaml
import glob
import os
import argparse
import time
import concurrent.futures
import re

from simHits import simHits as simHits_obj
from clusters import clusters as clusters_obj
from tracks import tracks as tracks_obj
from vertices import vertices as vertices_obj
from hits import hits as hits_obj
from pfparticles import pfparticles as pfparticles_obj

from tools import mergeCSVFiles
from generateConfig import generateConfig

def makeEvent(config_file, event=None):

    with open(config_file, 'r') as file:
        config = yaml.safe_load(file)


    nClusters = len(config['clusters']['centers'])
    nSamples = config['clusters']['n_samples']
    centers = config['clusters']['centers']

    bins = np.linspace(config['hits']['bins']['start'],
                       config['hits']['bins']['end'],
                       config['hits']['bins']['n_bins'])
    
    x, y, clusters = clusters_obj(config['clusters']['centers'],
                                  config['clusters']['std'],
                                  config['clusters']['n_samples'],
                                  bins,
                                  config['clusters']['pfparticles'], 
                                  config['clusters']['csv_file'],
                                  event).getHitsAndClusters()
    
    simHits_obj(clusters, bins, config['simHits']['csv_file'], event)
    hits_obj(x, y, bins, config['hits']['csv_file'], event)
    tracks_obj(centers,
               nClusters,
               nSamples,
               config['tracks']['charges'],
               config['tracks']['pfparticles'],
               config['tracks']['csv_file'],
               event)
    vertices_obj(centers,
                 nClusters,
                 config['vertices']['csv_file'],
                 event)
    pfparticles_obj(config['pfparticles']['charge'],
                    config['pfparticles']['csv_file'],
                    event)

def makeEventWrapper(args):
    makeEvent(args[0], args[1])

def mergeCSVFilesWrapper(args):
    mergeCSVFiles(args)

if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Generate samples')
    parser.add_argument('--nEvents', type=int, default=1)
    parser.add_argument('--nWorkers', type=int, default=1)
    parser.add_argument('--report', action='store_true')
    args = parser.parse_args()

    config_dir = os.environ['CONFIG_DIRECTORY']
    if not os.path.exists(config_dir):
        os.makedirs(config_dir)
    csv_dir = os.environ['CSV_DIRECTORY']
    if not os.path.exists(csv_dir):
        os.makedirs(csv_dir)
    tmp_dir = csv_dir.replace('csv', 'tmp')
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)


    start = time.time()

    generateConfig(args.nEvents)
    
    stop = time.time()
    generateConfigDelta = stop - start

    start = time.time()
    
    yml_files = [config_dir+'/config_%d.yml' % k for k in range(args.nEvents)]
    with concurrent.futures.ProcessPoolExecutor(args.nWorkers) as executor:
        for k, yml_file in enumerate(yml_files):
            executor.submit(makeEventWrapper, [yml_file, k])

    stop = time.time()
    makeEventDelta = stop - start

    start = time.time()
    
    csv_files = glob.glob(tmp_dir+'/*.csv')
    csv_files = [csv_file.split('/')[-1] for csv_file in csv_files]
    pattern = r'_[0-9]*.?\.csv'
    labels = [re.sub(pattern, '', csv_file) for csv_file in csv_files]
    labels = list(set(labels))

    with concurrent.futures.ProcessPoolExecutor(len(labels)) as executor:
        for label in labels:
            executor.submit(mergeCSVFilesWrapper, label)

    stop = time.time()
    mergeDelta = stop - start

    # remove tmp folder
    tmp_files = glob.glob(tmp_dir+'/*')
    for tmp_file in tmp_files:
        os.remove(tmp_file)
    os.rmdir(tmp_dir)

    if (args.report == True):
        print("=== Timing Report ===")
        print(" generateConfigDelta: %f" % generateConfigDelta)
        print(" makeEventDelta: %f" % makeEventDelta)
        print(" mergeDelta: %f" % mergeDelta)
        print("=====================")

    

    #for k in range(args.nEvents):
    #    generateConfig(k, args.seed)
    #    #print("Generating event %d from file %s" % (k, str(yml_file)))
    #    makeEvent('config/config_%d.yml' % k) 
