/**
 * @brief The DataLoader class is responsible for loading data from ROOT files or CSV files into memory.
 * 
 * The DataLoader class provides methods to fill various data structures such as HitsSoA, SimCluster, SimTrack, and SimVertex.
 * It also provides methods to retrieve the loaded data.
 * 
 * The DataLoader class can load data from a single ROOT file or multiple ROOT files, and it supports loading data from CSV files
 * using the ROOT framework's RDF (RDataFrame) library.
 * 
 * Usage:
 * 1. Create an instance of the DataLoader class.
 * 2. Call the appropriate fill methods to load the desired data into memory.
 * 3. Use the getter methods to retrieve the loaded data.
 * 
 * Example:
 * ```
 * DataLoader dataLoader;
 * dataLoader.fillSimHitsSoA();
 * const std::vector<HitsSoA>& simHitsSoA = dataLoader.getSimHitsSoA();
 * // Use the loaded data...
 * ```
 */

#ifndef DataLoader_h
#define DataLoader_h

#include "HitsSoA.h"
#include "SimCluster.h"
#include "SimTrack.h"
#include "SimVertex.h"
#include "PFParticle.h"
#include "TBranch.h"
#include "TFile.h"
#include "TTree.h"
#include <ROOT/RCsvDS.hxx>
#include <ROOT/RSnapshotOptions.hxx>

#include <vector>
#include <memory>
#include <string>
#include <unordered_map>

typedef std::vector<std::vector<uint32_t>> hits_in_sc;
typedef std::vector<std::vector<float>> fractions_in_sc;

class DataLoader {

public:
  DataLoader();
  DataLoader(std::shared_ptr<TFile> file);
  DataLoader(std::vector<std::string> files);

  void fillPFParticles();
  void fillSimHitsSoA();
  void fillHitsSoA();
  void fillSimClusters();
  void fillSimVertices();
  void fillSimTracks();
  void writeMergedSimClusters(std::vector<std::vector<SimCluster>>& merged_scs);

  const std::vector<HitsSoA> &getSimHitsSoA() { return simHitsSoA_; }
  const std::vector<HitsSoA> &getHitsSoA() { return hitsSoA_; }
  const std::vector<std::vector<SimCluster>> &getSimClusters() { return simClusters_; }
  const std::vector<std::vector<SimTrack>> &getSimTracks() { return simTracks_; }
  const std::vector<std::vector<SimVertex>> &getSimVertices() { return simVertices_; }
  const std::vector<std::vector<PFParticle>> &getPFParticles() { return pfParticles_; }

private:
  std::shared_ptr<TFile> input_file_;
  std::vector<std::string> input_files_v_;
  std::vector<std::vector<PFParticle>> pfParticles_;
  std::vector<HitsSoA> simHitsSoA_;
  std::vector<HitsSoA> hitsSoA_;
  std::vector<std::vector<SimCluster>> simClusters_;
  std::vector<std::vector<SimTrack>> simTracks_;
  std::vector<std::vector<SimVertex>> simVertices_;

  std::unordered_map<uint32_t, std::pair<float, float>> cached_position_;

  template <typename T>
  std::vector<std::vector<T>> fillVector(const char *treeName,
                                         const char *branchName, std::shared_ptr<TFile> file, 
                                         bool isFlatDataFrame);

  void loadFromCSV(std::string csv_file, 
                   std::string treeName, 
                   std::string root_file,
                   ROOT::RDF::RSnapshotOptions snapopts);

};

#endif
