/**
 * @class HitMergeVertex
 * @brief Represents a vertex that merges hits from multiple clusters.
 * 
 * The HitMergeVertex class is a subclass of MergeVertex<SimClusterMergeWrapper> and is used to merge hits from multiple clusters into a single vertex.
 * It provides methods for connecting vertices, updating energies, and absorbing hits from other vertices.
 * The class also includes member variables for storing hits and fractions, as well as energy calculations and scoring.
 
 */

#ifndef HitMergeVertex_h
#define HitMergeVertex_h

#include "Hits.h"
#include "MergeVertex.h"
#include "SimClusterMergeWrapper.h"

#include <algorithm>
#include <memory>


class HitMergeVertex : public MergeVertex<SimClusterMergeWrapper> {
public:
  HitMergeVertex(SimClusterMergeWrapper* t, 
                 const HitsSoA* hits, 
                 const std::vector<std::pair<uint32_t, float>>& hitsAndFractions);
  bool shouldConnect(const HitMergeVertex* rhs) const;
  void updateIsLow() override;
  void absorb(HitMergeVertex* rhs);

  inline float z() {
    if (valid())
      return objects().at(0)->z();
    return 0;
  }

  float energy(float x, float y) const;

  float calcScore(const HitMergeVertex* rhs) const;

  static float connectThreshold;
  static float singleClusterEfrac;

  bool operator==(const HitMergeVertex &rhs) const {
    return this == &rhs;
  }

private:
  void updateEnergies();

  struct HitAndFraction {
    HitAndFraction(const float& x, const float& y, const float& z, const float& e, const float& t, const float& frac, const uint32_t& detid) 
      : pos(Vector3D(x, y, z)), energy(e), time(t), fraction(frac), id(detid) {};

    Vector3D pos;
    float energy, time, fraction;
    uint32_t id;

    bool operator==(const HitAndFraction &rhs) const {
      return id == rhs.id;
    }
    bool operator<(const HitAndFraction &rhs) const {
      return id < rhs.id;
    }
    bool operator>(const HitAndFraction &rhs) const {
      return id > rhs.id;
    }
  };

  std::vector<HitAndFraction> hitsAndFractions_;
  mutable std::pair<const HitMergeVertex*, float> cached_score_;

  // total energy of the vertex, using fractions
  float energy_;
  // total energy of all hits assigned tot this vertex,
  // not including fractions
  float energyAllHits_;

  void absorbHitsAndFractions(const std::vector<HitAndFraction> &haf_rhs);
};

#endif
