/**
 * @file MergeGraph.h
 * @brief Contains the definition of the edge and MergeGraph classes.
 */

#ifndef MergeGraph_h
#define MergeGraph_h

#include "../interface/HitMergeVertex.h"
#include <algorithm>
#include <iostream>
#include <stdexcept>
#include <vector>
#include <memory>

/**
 * @class edge
 * @brief Represents an edge between two vertices in a graph.
 * 
 * The edge class stores information about the source vertex, target vertex, quality, and type of the edge.
 * It provides methods to update the edge, check its validity, and print debug information.
 * The edge class also defines a comparator for sorting edges based on their type and quality.
 * 
 * @tparam T The type of the vertices connected by the edge.
 */

template <class T> 
class edge {

  public:
    enum etype { lowToLow, highToLow, invalid };

    edge() : from_(0), to_(0), qual_(0.), type_(lowToLow) {} //default constructor
    edge(T* from, T* to, float qual) : from_(from), to_(to), qual_(qual) {
      updateType();
    }

    bool operator==(const edge &rhs) const {
      return from_ == rhs.from_ && to_ == rhs.to_ && qual_ == rhs.qual_ &&
            type_ == rhs.type_;
    }

    T* from() const { return from_; }
    T* to() const { return to_; }
    const float &qual() { return qual_; }
    bool valid() { return type != invalid; }

    void update(const T* hasabs = 0, const T* wasabs = 0) {
      qual_ = from()->calcScore(to());
    }

    void updateType() {
      if (!from_->shouldConnect(to_)) {
        type_ = invalid;
      } else if (from_->isLow() && to_->isLow()) {
        type_ = lowToLow;
      } else if (from_->isHigh() && to_->isLow()) {
        type_ = highToLow;
      } else {
        throw std::invalid_argument("Connection not allowed");
      }
    }
    const etype &type() { return type_; }

    void debug_print() const {
      std::string p =
          from_->name() + " to " + to_->name() + ": " + std::to_string(qual_);
      if (type_ == lowToLow)
        p += " lowToLow";
      else if (type_ == highToLow)
        p += " highToLow";
      else if (type_ == invalid)
        p += " invalid";
      std::cout << p << std::endl;
    }

    static bool comparator(edge &e0, edge &e1o) {
      if (e1o.type() == invalid)
        return true;
      if (e0.type() != e1o.type()) {
        if (e0.type() == lowToLow)
          return true; 
        return false; 
      }
      return e0.qual() > e1o.qual();
    }

  private:
    T *from_, *to_;
    float qual_;
    etype type_;
};

/**
 * @class MergeGraph
 * @brief Represents a merge graph.
 * 
 * The MergeGraph class stores a collection of vertices and edges.
 * It provides methods to set the vertices, add a vertex, create edges between vertices, merge the graph, and print the edges.
 * The MergeGraph class uses the edge class to represent the edges in the graph.
 * 
 * @tparam T The type of the vertices in the graph.
 */

template <class T> 
class MergeGraph {

  public:
    MergeGraph() {}

    void setVertices(const std::vector<T> v) {
      es_.clear();
      vs_ = v;
    }

    void addVertex(T &&v) {
      es_.clear();
      vs_.push_back(v);
    }

    void createEdges() {
      es_.clear();
      for (auto &vi : vs_) {
        if (!vi.valid())
          continue;
        for (auto &vj : vs_) {
          if (!vj.valid()) {
            continue;
          }
          if (vi.shouldConnect(&vj) && &vi != &vj) {
            es_.push_back(edge<T>(&vi, &vj, vi.calcScore(&vj)));
          }
        }
      }
    }

    int merge() {
      if (es_.size() < 1)
        return 0;
      int nmerged = 0;
      sortEdges();
      while (true) {
        if ((!es_.size()) || es_.at(0).type() == edge<T>::invalid)
          break;

        es_.at(0).from()->absorb(es_.at(0).to());
        bool changes = updateEdges(es_.at(0).from(), es_.at(0).to());
        if (!changes)
          break;
        nmerged++;
      }
      return nmerged;
    }

    void print_edges() {
      for (auto &e : es_) {
        e.debug_print();
      }
    }

    const std::vector<T> &vertices() const { return vs_; }
  private:
    void sortEdges() { std::sort(es_.begin(), es_.end(), edge<T>::comparator); }

    bool updateEdges(T *hasabs, T *wasabs) {
      std::vector<edge<T>> newedges, oldedges;

      for (const auto &e : es_) {
        if (e.to() != hasabs && e.to() != wasabs && e.from() != hasabs &&
            e.from() != wasabs)
          oldedges.push_back(e);
      }

      for (const auto &vi : {hasabs, wasabs}) {
        if (vi->valid()) {
          for (auto &vj : vs_) {
            if (vj.valid() && vi->shouldConnect(&vj) && vi != &vj) {
              newedges.push_back(edge<T>(vi, &vj, vi->calcScore(&vj)));
            }
          }
        }
      }

      for (auto& vi: vs_) {
        if (!vi.valid())
          continue;
        for (const auto& vj : {hasabs, wasabs}) {
          if (vj->valid() && vi.shouldConnect(vj) && &vi != vj) {
            newedges.push_back(edge<T>(&vi, vj, vi.calcScore(vj)));
          } 
        }
      }


      std::sort(newedges.begin(), newedges.end(), edge<T>::comparator);

      es_.clear();
      size_t newedgecounter = 0;
      for (size_t i = 0; i < oldedges.size(); ++i) {
        bool newfirst = false;
        if (newedgecounter < newedges.size())
          newfirst =
              edge<T>::comparator(newedges.at(newedgecounter), oldedges.at(i));
        if (newfirst) {
          es_.push_back(newedges.at(newedgecounter));
          newedgecounter++;
          i--;
        } else {
          es_.push_back(oldedges.at(i));
        }
      }
      
      while (newedgecounter < newedges.size()) {
        es_.push_back(newedges.at(newedgecounter));
        newedgecounter++;
      }

      return newedges.size() > 0;
    }

    std::vector<T> vs_;
    std::vector<edge<T>> es_;
};

#endif
