#ifndef RootLoader_h
#define RootLoader_h

#include "HitsSoA.h"
#include "SimShower.h"
#include "SimTrack.h"
#include "SimVertex.h"

#include "TFile.h"
#include "TTree.h"
#include "TTreeReader.h"
#include "TTreeReaderValue.h"
#include "TInterpreter.h"
#include "TFileMerger.h"

#include <memory>
#include <stdexcept>
#include <filesystem>
#include <cstdlib>
#include <string>

class ROOTLoader {
    public:
        ROOTLoader() {};
        ROOTLoader(const char* filename, const char* treeName) : filename_(filename), treeName_(treeName) {
            gInterpreter->GenerateDictionary("vector<vector<unsigned int>>", "vector");
            gInterpreter->GenerateDictionary("vector<vector<float>>", "vector");

            const std::filesystem::path path(filename_);
            if (std::filesystem::is_directory(path)) {
                TFileMerger merger;
                for (const auto& entry : std::filesystem::directory_iterator(path)) {
                    if (entry.path().extension() == ".root") {
                        merger.AddFile(entry.path().c_str());
                    }
                }
                merger.OutputFile((filename_ + std::string("/merged.root")).c_str());
                merger.Merge();
                filename_ = (filename_ + "/merged.root").c_str();
            }

            try {
                file_ = std::make_unique<TFile>(filename_.c_str(), "update");
            } catch (const std::exception& e) {
                throw std::runtime_error("Failed to initialize ROOTLoader: " + std::string(e.what()));
            }

            tree_ = (TTree*)file_->Get(treeName_);
            if (!tree_) {
                throw std::runtime_error("Failed to initialize ROOTLoader: tree not found");
            }

            nEvents_ = tree_->GetEntries();

            fillHitsSoA();
            fillSimShowers();
            fillSimTracks();
            fillSimVertices();
        }

    const std::vector<HitsSoA> &getHitsSoA() { return hitsSoA_; }
    const std::vector<SimShowerCollection> &getSimShowers() { return simShowers_; }
    const std::vector<SimTrackCollection> &getSimTracks() { return simTracks_; }
    const std::vector<SimVertexCollection> &getSimVertices() { return simVertices_; }
    
    Long64_t nEvents() { return nEvents_; }

    void saveMergedSimShowers(std::vector<SimShowerCollection>& merged_showers);
    private:
        std::unique_ptr<TFile> file_ = nullptr;
        TTree* tree_ = nullptr;
        std::string filename_;
        const char* treeName_;
        std::vector<HitsSoA> hitsSoA_;
        std::vector<SimShowerCollection> simShowers_;
        std::vector<SimTrackCollection> simTracks_;
        std::vector<SimVertexCollection> simVertices_;

        Long64_t nEvents_;

        void fillHitsSoA();
        void fillSimShowers();
        void fillSimTracks();
        void fillSimVertices();
       
       // vectors for mergedSimShowers data
        std::vector<float> posBoundary_x, posBoundary_y, posBoundary_z, 
                           impactPoint_x, impactPoint_y, impactPoint_z,
                           impactMomentum_x, impactMomentum_y, impactMomentum_z, impactMomentum_t,
                           energy;
        std::vector<std::vector<uint32_t>> hits;
        std::vector<std::vector<float>> fractions;
        std::vector<int> layer, pdgId;
        std::vector<uint32_t> event;

        void clear() {
            posBoundary_x.clear();
            posBoundary_y.clear();
            posBoundary_z.clear();
            impactPoint_x.clear();
            impactPoint_y.clear();
            impactPoint_z.clear();
            impactMomentum_x.clear();
            impactMomentum_y.clear();
            impactMomentum_z.clear();
            impactMomentum_t.clear();
            energy.clear();
            hits.clear();
            fractions.clear();
            layer.clear();
            pdgId.clear();
            event.clear();
        }

        template<typename T>
        std::vector<std::vector<T>> fillVector(const char* branchName) {
            TTreeReader reader(treeName_, file_.get());
            std::vector<std::vector<T>> result;

            TTreeReaderValue<std::vector<T>> reader_value(reader, branchName);
            while (reader.Next()) {
                result.push_back(*reader_value);
            }
            return result;
        }

        void setBranches() {
            tree_->Branch("mergedSimCluster_posBoundary_x", "vector<float>", &posBoundary_x);
            tree_->Branch("mergedSimCluster_posBoundary_y", "vector<float", &posBoundary_y);
            tree_->Branch("mergedSimCluster_posBoundary_z", "vector<float>", &posBoundary_z);
            tree_->Branch("mergedSimCluster_impactPoint_x", "vector<float>", &impactPoint_x);
            tree_->Branch("mergedSimCluster_impactPoint_y", "vector<float>", &impactPoint_y);
            tree_->Branch("mergedSimCluster_impactPoint_z", "vector<float>", &impactPoint_z);
            tree_->Branch("mergedSimCluster_impactMomentum_x", "vector<float>", &impactMomentum_x);
            tree_->Branch("mergedSimCluster_impactMomentum_y", "vector<float>", &impactMomentum_y);
            tree_->Branch("mergedSimCluster_impactMomentum_z", "vector<float>", &impactMomentum_z);
            tree_->Branch("mergedSimCluster_impactMomentum_t", "vector<float>", &impactMomentum_t);
            tree_->Branch("mergedSimCluster_energy", "vector<float>", &energy);
            tree_->Branch("mergedSimCluster_layer", "vector<int>", &layer);
            tree_->Branch("mergedSimCluster_pdgId", "vector<int>", &pdgId);
            tree_->Branch("mergedSimCluster_hits", "vector<vector<unsigned int>>", &hits);
            tree_->Branch("mergedSimCluster_fractions", "vector<vector<float>>", &fractions);
            tree_->Branch("mergedSimCluster_event", "vector<unsigned int>", &event);
        }

        void setBranches(TTree* tree) {
            tree->Branch("mergedSimCluster_posBoundary_x", "vector<float>", &posBoundary_x);
            tree->Branch("mergedSimCluster_posBoundary_y", "vector<float", &posBoundary_y);
            tree->Branch("mergedSimCluster_posBoundary_z", "vector<float>", &posBoundary_z);
            tree->Branch("mergedSimCluster_impactPoint_x", "vector<float>", &impactPoint_x);
            tree->Branch("mergedSimCluster_impactPoint_y", "vector<float>", &impactPoint_y);
            tree->Branch("mergedSimCluster_impactPoint_z", "vector<float>", &impactPoint_z);
            tree->Branch("mergedSimCluster_impactMomentum_x", "vector<float>", &impactMomentum_x);
            tree->Branch("mergedSimCluster_impactMomentum_y", "vector<float>", &impactMomentum_y);
            tree->Branch("mergedSimCluster_impactMomentum_z", "vector<float>", &impactMomentum_z);
            tree->Branch("mergedSimCluster_impactMomentum_t", "vector<float>", &impactMomentum_t);
            tree->Branch("mergedSimCluster_energy", "vector<float>", &energy);
            tree->Branch("mergedSimCluster_layer", "vector<int>", &layer);
            tree->Branch("mergedSimCluster_pdgId", "vector<int>", &pdgId);
            tree->Branch("mergedSimCluster_hits", "vector<vector<unsigned int>>", &hits);
            tree->Branch("mergedSimCluster_fractions", "vector<vector<float>>", &fractions);
            tree->Branch("mergedSimCluster_event", "vector<unsigned int>", &event);
        }

        template<typename T>
        void addBranch(const char* name, std::vector<std::vector<T>>& data) {
            TBranch* branch = tree_->Branch(name, &data);
            branch->SetAddress(&data);
            for (Long64_t i = 0; i < data.size(); ++i) {
                tree_->GetEntry(i);
                tree_->Fill();
            }
            tree_->Write();
        }
};
#endif
