#ifndef SimCluster_h
#define SimCluster_h

#include "HitsSoA.h"
#include "SimTrack.h"
#include "Tools.h"
#include <algorithm>
#include <cstdint>
#include <limits>
#include <utility>
#include <vector>
#include <iostream>

/**
 * @class SimCluster
 * @brief Represents a simulated cluster of hits in a detector.
 *
 * The SimCluster class stores information about a simulated cluster, including its impact momentum,
 * position at the boundary, impact point, energy, layer, hits, fractions, and PDG ID.
 * It provides methods to access and modify these properties.
 */
class SimCluster {
public:
  SimCluster() {
    impactMomentum_ = Vector4D();
    positionAtBoundary_ = Vector3D();
    impactPoint_ = Vector3D();
    energy_ = 0.f;
    cluster_width_x_ = 0.f;
    cluster_width_y_ = 0.f;
    cluster_width_z_ = 0.f;
    layer_ = std::numeric_limits<unsigned int>::max();
    pdgId_ = std::numeric_limits<int>::max();
  }

  SimCluster(Vector4D impactMom, Vector3D posBoundary, Vector3D impactPoint,
             float energy, float cluster_width_x, float cluster_width_y, float cluster_width_z,
             unsigned int layer, std::vector<uint32_t> hits,
             std::vector<float> fractions, int pdgId, int genParticle)
      : impactMomentum_(impactMom), positionAtBoundary_(posBoundary),
        impactPoint_(impactPoint), energy_(energy), cluster_width_x_(cluster_width_x), cluster_width_y_(cluster_width_y), cluster_width_z_(cluster_width_z),
        layer_(layer), pdgId_(pdgId), genParticle_(genParticle) {
      unsigned int nHits = hits.size();
      hits_.resize(nHits);
      fractions_.resize(nHits);
      for (unsigned int i = 0; i < nHits; ++i) {
        if (std::find(hits_.begin(), hits_.end(), hits[i]) != hits_.end())
          continue;
        hits_[i] = hits[i];
        fractions_[i] = fractions[i];
      }
  }    

  Vector3D positionAtBoundary() { return positionAtBoundary_; }
  Vector3D impactPoint() { return impactPoint_; }
  Vector4D impactMomentum() { return impactMomentum_; }

  void setImpactMomentum(Vector4D impactMomentum) {
    impactMomentum_ = impactMomentum;
  }

  std::vector<std::pair<uint32_t, float>> hits_and_fractions() const {
    std::vector<std::pair<uint32_t, float>> result;
    auto nHits = hits_.size();
    for (unsigned int i = 0; i < nHits; ++i) {
      result.push_back(std::make_pair(hits_[i], fractions_[i]));
    }
    return result;
  }

  void addRecHitAndFraction(uint32_t hit, float fraction) {
    std::vector<uint32_t>::iterator it =
        std::find(hits_.begin(), hits_.end(), hit);
    if (it == hits_.end()) {
      hits_.push_back(hit);
      fractions_.push_back(fraction);
    } else {
      int i = std::distance(hits_.begin(), it);
      fractions_[i] += fraction;
    }
  }

  float energy() { return energy_; }
  unsigned int getLayer() const { return layer_; }

  float radius_xy() { return std::sqrt(cluster_width_x_ * cluster_width_x_ + cluster_width_y_ * cluster_width_y_); }
  
  void setPdgId(int pdgId) { pdgId_ = pdgId; }
  int pdgId() const { return pdgId_; }

  int genParticle() const { return genParticle_; }

  void addG4Tracks(const SimTrack &t);
  const std::vector<SimTrack> &g4Tracks() const { return g4Tracks_; }

  SimCluster operator+(SimCluster &);
  SimCluster operator+=(SimCluster &);

private:
  std::vector<SimTrack> g4Tracks_;
  Vector4D impactMomentum_;
  Vector3D positionAtBoundary_;
  Vector3D impactPoint_;
  float energy_;
  float cluster_width_x_, cluster_width_y_, cluster_width_z_;
  unsigned int layer_;
  std::vector<uint32_t> hits_;
  std::vector<float> fractions_;
  int pdgId_;
  int genParticle_;
};

#endif
