#!/bin/sh

usage="$(basename "$0") [-h] [-r] [-t] [-g] [-m] NWORKERS NEVENTS -- script to run the workflow

where:
    -h  show this help text
    -r  recompile the project
    -t  run the tests
    -g  generate the samples
    -m  run the merging algorithm

  NEVENTS  is the number of events to be generated
  NWORKERS is the number of processors to be used for the data generation
  "

recompile=false
generate_samples=false
run_merging=false
run_tests=false
while getopts "rtgmh" opt; do
  case $opt in
    r) recompile=true ;;
    t) run_tests=true ;;
    g) generate_samples=true ;;
    m) run_merging=true ;;
    h) echo "$usage"
       exit
       ;;
  esac
done

print_step() {
    echo ">>> ` date` $1"
}

parse_yml() {
    python3 -c "import yaml; print(yaml.safe_load(open('data/config.yml')))"
}

cat << EOF

=================================================================================

                                        :::::                                                   
                                   ::::::: :::..                                                
                                ::*#::       :::       %                                        
                             ::: :  :=-.      ::      #                                         
                          .::.  : ::::#=*    ::.     -* % *                                     
                        :::   :::       #:-  -#.  -  # % *#                                     
                      ::.  ::     *     .:*:+:@ --- @ @+**                                      
                    ::.  :.        .=  - .:#### -* @ @@@                                        
                  ::.  :             :=:  -: @##@ *@@@@@                                        
                .::  :             -+#.#:=-  @:@#@@@@@+                                         
               ::  :            =:=@@%@@-#= - -@@@@@@---                                        
             ::  ::           #-#:@::@#=@@:#% @ @@@@#@-                                         
            ::  :          .==-#-: -@-=@@.==@#-%%@--###:                                        
          ::  .:          +=.% :.  ::*@@@%-@@@@@@    :#**                                       
         ::  :.         +--@ ::  :: -+@@- @@-@@@#:     =##+                                     
        ::  :         --=+:::  ::  #@#*##@-@@@#.=:#      ##*:                                   
       ::#:: =       *..#::  ::  @%#*=:#: @@@%@  -=:+      ##*:                                 
      #*:  :   +    *-.*   ::  *##*##:#.-:@@@= %*  + *:     :#+=                                
     ::::#:     :: +:+# .=: :=@#:##::::- @@#::@= @=  =.*:     -#+: .:::::::.:                   
    :-   #-#    ::#=.#:+::%@@*#=:*:::=@@*@* :#  @ .@. .=:#      ##:-       : :                  
     :    ::#:   -=:#:@@@@@  @###:   :#@@#:.+ :  =@ *%  --*%==::+ :::.      ::                  
   ..:      =:# *--:-+#.=  =@ -#  : =::@@@%:   :###%= @@@@@+#:*+   .-:-    :::                  
    :::     ::###:     :=:=  %= %= #: @@*:= .#:::::+@++ :::+=.#:      +::  ::                   
     ::::::::  :*##      =:-. :@: @+#@@ - :=::+#:#%=  ::  #+:=  :     ::::::                    
                 =##=     .# =  -@ @@@ :=:-:##:@#=  ::  :%-:=    .:  :   #:                     
                   *##:     :* =  @@@@# =*:::-** .::  ::@=.=       :::= ::                      
                    :###:     =+@@=@@+ %@#-#@+  ::  ::====         .:  ::                       
                      :*##    %-@@@@ :@:-@@=- ::  :  +=--         ::  ::                        
                        =##-% #**@#:=  -@@+@:%  ::.@:-=          :  ::                          
                         .@*#@:%%@::+:= #@@:-% :=+=-=           :  ::                           
                        %  :@#@@@@   :++@=:@#+@-=-=           :  ::                             
                      .+  -@.@@@@@+   -==*=-*=-=.           ::  ::                              
                     %   -%-@@@@@@=   :#.=:%=.            ::  ::                                
                   -#   -%:@@@@@ #*#::: :*::            :.  ::                                  
                  +   -*= @@-@@   :+##-  :  .:        :   ::                                    
                     -%  %%%@#     ::::+..    ::   ::  .::                                      
                     %  %%%%#     ::   +:=      ::   :::                                        
                       %%%%-.     .:     ::::. :. :::                                           
                      % % .*     ::.      :::::::::                                             
                                 : :        ::::                                                
                                  ::::::::::                                                    
               ____   ______ ______ ____   __  __ ______ __  __
              / __ \ / ____//_  __// __ \ / / / //_  __// / / /
             / /_/ // /_     / /  / /_/ // / / /  / /  / /_/ / 
            / ____// __/    / /  / _, _// /_/ /  / /  / __  /  
           /_/    /_/      /_/  /_/ |_| \____/  /_/  /_/ /_/   
                                                              
=================================================================================

EOF


print_step "Sourcing the LCG stack"
source /cvmfs/sft.cern.ch/lcg/views/LCG_105/x86_64-el9-gcc11-opt/setup.sh

export CONFIG_DIRECTORY="/ceph/abrusamolino/PFTruth/config-test"
export CSV_DIRECTORY="/ceph/abrusamolino/PFTruth/csv-test"

NWORKERS=${2:-4}
NEVENTS=${3:-0}

CURRENT_DIR=$(pwd)

# Run the data generation script
if $generate_samples ; then
    print_step "Generating the data"
    cd data
    python3 makeSamples.py --nEvents $NEVENTS --nWorkers $NWORKERS --report
fi

cd $CURRENT_DIR
# Compile the project if executable does not exist
if $recompile ; then
    print_step "Cleaning up before recompiling"
    rm -rf build
    print_step "Generating build files"
    mkdir -p build && cd build
    print_step "Running the tests"
    cmake -DBUILD_TESTING=ON ..
    print_step "Compiling the project"
    make -s -j $NWORKERS
    cd ${CURRENT_DIR}
    print_step "Generating the documentation"
    doxygen Doxyfile
fi

score_thresholds=(0.1 0.2 0.3 0.4 0.5)
score_labels=("0p1" "0p2" "0p3" "0p4" "0p5")
frac_thresholds=(0.95 0.85 0.75 0.65)
frac_labels=("0p95" "0p85" "0p75" "0p65")

if $run_merging ; then
  print_step "Running the cluster merging"
  for i in "${!frac_thresholds[@]}"; do
    for j in "${!score_thresholds[@]}"; do
      INPUT_FILE="/ceph/abrusamolino/PFTruthSamples/COCOA_wBackscattering/SingleElectron_Geant4_FlatEtaPt.root"
      OUTPUT_FILE="/ceph/abrusamolino/PFTruthSamples/COCOA_wBackscattering/SingleElectron_Geant4_FlatEtaPt_${frac_labels[i]}_${score_labels[j]}.root"
      ./build/COCOATruth ${INPUT_FILE} ${frac_thresholds[i]} ${score_thresholds[j]}
      cp ${INPUT_FILE} ${OUTPUT_FILE}
      print_step "Saved output file to ${OUTPUT_FILE}"
    done
  done
fi

print_step "Done"

