#include "../interface/SimHistoryTool.h"
#include <algorithm>
#include <numeric>

SimCluster SimHistoryTool::createMergedSimCluster(
    std::vector<SimCluster *> tomerge) const {
  SimCluster out;

  if (tomerge.size() < 1)
    return out;
  if (tomerge.size() == 1)
    return *tomerge.at(0);

  // Vector4D p4, point;
  for (const auto &sc : tomerge) {
    out += *sc;
  }

  out.setPdgId(createMergedSimClusterID(&out));
  return out;
}

const SimTrack *SimHistoryTool::traverseStep(const SimTrack *st) const {
  const SimTrack *out = st;
  int vidx = out->vertIndex();
  if (vidx < 0)
    return out; // no vertex
  const SimVertex &vertex = simvertices_.at(vidx);
  int stid = vertex.parentIndex();
  if (stid < 0)
    return out; // no parent
  int stidx = trackIdToTrackIdxAssociation_.at(stid);
  if (stidx < 0)
    return out;
  const SimTrack &simtrack = simtracks_.at(stidx);
  vidx = simtrack.vertIndex();
  out = &simtrack;
  return out;
}

std::vector<const SimTrack *> SimHistoryTool::createHistory(const SimTrack *st) const {
  std::vector<const SimTrack *> outv;
  const SimTrack *out = st;
  outv.push_back(st);
  while (true) {
    const SimTrack *test = traverseStep(out);
    if (test == out || test == 0)
      break;
    out = test;
    outv.push_back(test);
  }
  return outv;
}

bool SimHistoryTool::inHistory(const SimTrack &possibleParent,
                               const SimTrack *probe) const {
  if (simTrackEqual(*probe, possibleParent))
    return true;
  while (true) {
    const SimTrack *test = traverseStep(probe);
    if (test == probe || test == 0)
      return false;
    if (simTrackEqual(*test, possibleParent))
      return true;
    probe = test;
  }
}

int SimHistoryTool::createMergedSimClusterID(SimCluster *sc,
                                             double idenfrac) const {
  int mergedPdgId = 0;
  double totalEnergy = sc->impactMomentum().E();

  const auto sts = sc->g4Tracks();
  std::vector<std::vector<const SimTrack *>> scHists;
  std::vector<const SimTrack *> allst;

  for (const auto &st : sts) {
    scHists.emplace_back(createHistory(&st));
    const auto &last = scHists.at(scHists.size() - 1);
    for (const auto &lst : last) {
      if (std::find(allst.begin(), allst.end(), lst) == allst.end())
        allst.push_back(lst);
    }
  }

  std::vector<float> sortEnIntoGroup(allst.size(), 0.);
  std::iota(sortEnIntoGroup.begin(), sortEnIntoGroup.end(), 0);
  std::sort(sortEnIntoGroup.begin(), sortEnIntoGroup.end(),
            [&](std::size_t i, std::size_t j) {
              return sortEnIntoGroup[i] > sortEnIntoGroup[j];
            });

  mergedPdgId = 0;
  for (const auto &idx : sortEnIntoGroup) {
    float stE = allst.at(idx)->momentum().E();
    float gE = sortEnIntoGroup.at(idx);
    float motherFracE = gE / stE;

    if (gE / totalEnergy < idenfrac)
      break;
    if (motherFracE > idenfrac) {
      mergedPdgId = allst.at(idx)->type();
    }
  }
  return mergedPdgId;
}

bool SimHistoryTool::simTrackEqual(const SimTrack &a, const SimTrack &b) {
  bool trkid = a.trackId() == b.trackId();
  bool evid = a.eventId() == b.eventId();
  bool vid = a.vertIndex() == b.vertIndex();
  return trkid && evid && vid;
}

const SimTrack *SimHistoryTool::getRoot(const SimTrack *st) const {
  const SimTrack *out = st;
  while (true) {
    const SimTrack *test = traverseStep(out);
    if (test == out || test == 0)
      break;
    out = test;
  }
  return out;
}

void SimHistoryTool::createIdxAssociation() {
  trackIdToTrackIdxAssociation_.clear();
  for (size_t i = 0; i < simtracks_.size(); ++i) {
    trackIdToTrackIdxAssociation_[simtracks_[i].trackId()] = i;
  }
}
